package models

import (
	"eta/eta_index_lib/utils"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type BaseFromSciData struct {
	SciDataId          int `orm:"column(sci_data_id);pk"`
	BaseFromSciIndexId int
	IndexCode          string
	DataTime           string
	Value              string
	CreateTime         time.Time
	ModifyTime         time.Time
	DataTimestamp      int64
}

// Update 修改
func (r *BaseFromSciData) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(r, updateCols...)

	return
}

// GetIndexDataList 获取所有的指标数据
func (r *BaseFromSciData) GetIndexDataList(indexCode string) (items []*BaseFromSciData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci_data WHERE 1=1 AND index_code = ? `
	_, err = o.Raw(sql, indexCode).QueryRows(&items)

	return
}

// BatchAdd 批量添加指标
func (r *BaseFromSciData) BatchAdd(list []*BaseFromSciData) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(len(list), list)

	return
}

func GetBaseFromSciDataByCondition(condition string, pars []interface{}) (list []*BaseFromSciData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

// AddEdbDataFromSci 新增卓创（红桃3）指标数据
func AddEdbDataFromSci(edbCode string) (err error) {
	o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	sciBaseDataAll, err := GetBaseFromSciDataByCondition(condition, pars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_sci(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range sciBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

// RefreshEdbDataFromSci 刷新卓创（红桃3）指标数据
func RefreshEdbDataFromSci(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_SCI
	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	sciDataList, err := GetBaseFromSciDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_sci(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range sciDataList {
		item := v

		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value
			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromSci add Err", err.Error())
			return
		}
	}
	return
}

// HandleSciExcelDataReq 卓创（红桃3）的excel数据
type HandleSciExcelDataReq struct {
	DataMap          map[string]map[string]string
	IndexNameList    []string
	ThirdIndexIdList []string
	FrequencyList    []string
	UnitList         []string
}

// BaseFromSciIndex 红桃3指标表格
type BaseFromSciIndex struct {
	BaseFromSciIndexId int       `orm:"column(base_from_sci_index_id);pk"  json:"base_from_sci_index_id"` //序号
	ClassifyId         int       `gorm:"column:classify_id" json:"classify_id"`
	IndexCode          string    `gorm:"column:index_code" json:"index_code" description:"指标编码"`
	IndexName          string    `gorm:"column:index_name" json:"index_name" description:"指标名称"`
	Frequency          string    `gorm:"column:frequency" json:"frequency"`
	Unit               string    `gorm:"column:unit" json:"unit"`
	StartDate          time.Time `gorm:"column:start_date" json:"start_date"`
	EndDate            time.Time `gorm:"column:end_date" json:"end_date"`
	CreateTime         time.Time `gorm:"autoCreateTime;column:create_time" json:"create_time"`       //创建时间
	ModifyTime         time.Time `gorm:"autoUpdateTime:milli;column:modify_time" json:"modify_time"` //最后更新时间
}

// TableName get sql table name.获取数据库表名
func (r *BaseFromSciIndex) TableName() string {
	return "base_from_sci_index"
}

// Update 修改
func (r *BaseFromSciIndex) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(r, updateCols...)

	return
}

// GetAllIndex 获取所有的指标
func (r *BaseFromSciIndex) GetAllIndex() (items []*BaseFromSciIndex, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_sci_index WHERE 1=1 `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

// BatchAdd 批量添加指标
func (r *BaseFromSciIndex) BatchAdd(list []*BaseFromSciIndex) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(len(list), list)

	return
}
