package utils

import (
	"encoding/json"
	"github.com/beego/beego/v2/core/logs"
	"github.com/sirupsen/logrus"
	"gopkg.in/natefinch/lumberjack.v2"
	"os"
	"path"
)

const (
	DefaultLogPath    = "./etalogs/filelog"
	DefaultBinlogPath = "./etalogs/binlog"
	DefaultApiLogPath = "./etalogs/apilog"
)

var FileLog = logrus.New()
var ApiLog = logrus.New()
var Binlog *logs.BeeLogger

func init() {
	if LogMaxDays == 0 {
		LogMaxDays = 30
	}
	logPath := LogPath
	if logPath == "" {
		logPath = DefaultLogPath
	}
	logFile := LogFile
	if logFile == "" {
		logFile = "filelog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	logConf := getDefaultLogrusConfig(logFileName)
	// 使用滚动压缩方式记录日志
	rolling(FileLog, logConf)
	//rolling(bLogFileName)
	// 设置日志输出JSON格式
	jsonFormat := new(logrus.JSONFormatter)
	jsonFormat.DisableHTMLEscape = true
	jsonFormat.TimestampFormat = HlbFormatDateTime
	FileLog.SetFormatter(jsonFormat)
	FileLog.SetReportCaller(true)
	//LogInstance.SetFormatter(&logrus.TextFormatter{})
	// 设置日志记录级别
	//FileLog.SetLevel(logrus.DebugLevel)

	//FileLog.Info("abc")
	initBinlog()
	initApiLog()
}

type logConfig struct {
	FileName string `json:"filename" description:"保存的文件名"`
	MaxLines int    `json:"maxlines"  description:"每个文件保存的最大行数，默认值 1000000"`
	MaxSize  int    `json:"maxsize" description:"每个文件保存的最大尺寸，默认值是 1 << 28, //256 MB"`
	Daily    bool   `json:"daily" description:"是否按照每天 logrotate，默认是 true"`
	MaxDays  int    `json:"maxdays" description:"文件最多保存多少天，默认保存 7 天"`
	Rotate   bool   `json:"rotate" description:"是否开启 logrotate，默认是 true"`
	Level    int    `json:"level" description:"日志保存的时候的级别，默认是 Trace 级别"`
	Color    bool   `json:"color" description:"日志是否输出颜色"`
	//Perm     string `json:"perm" description:"日志文件权限"`
}

func initBinlog() {
	//binlog日志
	//binlog日志
	binlogPath := BinLogPath
	if binlogPath == "" {
		binlogPath = DefaultBinlogPath
	}
	binlogFile := BinLogFile
	if binlogFile == "" {
		binlogFile = "binlog.log"
	}
	os.MkdirAll(binlogPath, os.ModePerm)
	logFileName := path.Join(binlogPath, binlogFile)
	Binlog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	logConf.MaxLines = 10000000
	logConf.Rotate = true
	b, _ := json.Marshal(logConf)
	Binlog.SetLogger(logs.AdapterFile, string(b))
	Binlog.EnableFuncCallDepth(true)
}

func initApiLog() {
	logPath := ApiLogPath
	if logPath == "" {
		logPath = DefaultApiLogPath
	}
	logFile := ApiLogFile
	if logFile == "" {
		logFile = "apilog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	logConf := getDefaultLogrusConfig(logFileName)
	// 使用滚动压缩方式记录日志
	rolling(ApiLog, logConf)
	//rolling(bLogFileName)
	// 设置日志输出JSON格式
	jsonFormat := new(logrus.JSONFormatter)
	jsonFormat.DisableHTMLEscape = true
	jsonFormat.TimestampFormat = HlbFormatDateTime
	ApiLog.SetFormatter(jsonFormat)
}

// 日志滚动设置
func rolling(fLog *logrus.Logger, config *lumberjack.Logger) {
	// 设置输出
	fLog.SetOutput(config)
}

func getDefaultLogrusConfig(logFile string) (config *lumberjack.Logger) {
	config = &lumberjack.Logger{
		Filename:   logFile,    //日志文件位置
		MaxSize:    256,        // 单文件最大容量,单位是MB
		MaxBackups: 0,          // 最大保留过期文件个数
		MaxAge:     LogMaxDays, // 保留过期文件的最大时间间隔,单位是天
		Compress:   true,       // 是否需要压缩滚动日志, 使用的 gzip 压缩
		LocalTime:  true,
	}
	return
}

func getDefaultLogConfig() logConfig {
	return logConfig{
		FileName: "",
		MaxLines: 0,
		MaxSize:  1 << 28,
		Daily:    true,
		MaxDays:  LogMaxDays, //我就是喜欢31天，咋滴，不喜欢你就自己改-_-!
		Rotate:   true,
		Level:    logs.LevelTrace,
		//Perm:     "",
	}
}
